/* -*- mode: c++ -*-
 * Header    :
 * File      : graphvizDataAnalizer.cxx
 * Auther    : Kosuke Shinoda
 * Since     : <05/06/29 11:28:06 kshinoda>
 * Time-stamp: <08/02/18 12:20:40 kshinoda>
 * Comment   :
 * End       :
 */
/*
 * ScaleFreeNetwork Simulator
 * Copyright (C)2005 SHINODA, Kosuke. NDA, JAPAN
 */

//#include "itk/itk.h"
#include "graphvizDataAnalizer.hxx"

#include <boost/graph/breadth_first_search.hpp>
#include <boost/graph/graphviz.hpp>

#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <map>
#include <vector>

using namespace GraphvizDataAnalizer;
using namespace boost;
//using namespace Itk;
using namespace std;

//--------------------------------------------------
void Analizer::init(GraphvizGraph g_dot) {
  GVIter vi, vi_end;
  for(tie(vi, vi_end) = vertices(g_dot); vi != vi_end; vi++) {
    add_vertex(*vi, g);
  }
  GEIter ei, ei_end;
  for(tie(ei, ei_end) = edges(g_dot); ei != ei_end; ++ei) {
    add_edge(source(*ei, g_dot), target(*ei, g_dot), g);
  } 

  n_nodes = num_vertices(g);
  n_edges = num_edges(g);
};

void Analizer::calc_NetworkProperties() {

  ScoreTable score(num_vertices(g));
  VIter vi, vi_end;
  int diameter = 0;

  for(tie(vi, vi_end) = vertices(g); vi != vi_end; vi++) {
    
    //########## diameter, path length ##########
    DistTable dist(num_vertices(g), 0);
    breadth_first_search(g, *vi, visitor(distance_recorder(&dist[0])));
    int count = 0;
    VIter vi2, vi2_end;
    for(tie(vi2, vi2_end) = vertices(g); vi2 != vi2_end; vi2++) {
      //ITK_DBG(dist[*vi2]);
      if(*vi != *vi2 && (int)dist[*vi2] == 0)
	count += 0;//(int)num_vertices(g);
      else
	count += (int)dist[*vi2];
      
      if(diameter < dist[*vi2])
	diameter = (int)dist[*vi2];
    }
    score[*vi] = (float)count / (float)(num_vertices(g) - 1);
    //ITK_DBG(score[*vi]);
    //exit(0);
  }
  
  float meanDist = 0.0;
  for(tie(vi, vi_end) = vertices(g); vi != vi_end; vi++) {
    //ITK_DBG(score[*vi]);
    meanDist += (score[*vi]/(float)num_vertices(g));
  }

  cout << (int)num_vertices(g) << " "<< (int)num_edges(g)<< " "<< meanDist << " ";

  calc_ClusteringCoefficient();

  cout << diameter << endl;
};

void Analizer::calc_ClusteringCoefficient() {

  VIter vi, vi_end;
  ScoreTable c(num_vertices(g));

  for(tie(vi,vi_end) = vertices(g); vi != vi_end; ++vi) {
    AIter ai, ai_end, ai2;
    tie(ai, ai_end) = adjacent_vertices(*vi, g);
    float adjacent_num = (float)std::distance(ai, ai_end);
    c[*vi] = 0.0;
    for(; ai != ai_end; ++ai) {
      ai2 = ai + 1;
      for(; ai2 != ai_end; ++ai2) {
	if(edge(*ai, *ai2, g).second == true)
	  c[*vi] += 2.0;
      }
    }
    if(adjacent_num != 0.0 && c[*vi] != 0.0)
      c[*vi] /= adjacent_num * (adjacent_num -1);
  }

  float meanClustringCoefficient = 0.0;
  for(tie(vi, vi_end) = vertices(g); vi != vi_end; vi++) {
    meanClustringCoefficient += (c[*vi] / (float)num_vertices(g));
  }

  cout << meanClustringCoefficient << " ";

};

void Analizer::output_DegreeDistribution() {

  stringstream filename;
  //--------------------------------------------------
  // ϼʬۤν񤭽Ф
  filename << "DegreeDistribution-an" << n_nodes << "-" << n_edges << ".dat";
  ofstream data_file(filename.str().c_str());
  map<int, int> data_table;
  VIter it, it_end;
  
  for(tie(it, it_end) = vertices(g); it != it_end; ++it) 
    data_table[out_degree(*it, g)] += 1;
  
  for(map<int,int>::iterator mi = data_table.begin(); mi != data_table.end(); mi++) 
    if((*mi).first != 0)
      data_file << (*mi).first << " " << ((*mi).second/(float)n_nodes) << endl;

  data_file.close();

}

//--------------------------------------------------
void Analizer::output_Closeness() {
  stringstream filename;
  filename << "network_closeness-an" << n_nodes << "-" << n_edges << ".dat";
  ofstream data_file(filename.str().c_str());

  ScoreTable score(n_nodes);
  score = calc_Closeness();
  VIter it, it_end;
  for(tie(it, it_end) = vertices(g); it != it_end; ++it) 
    data_file << (int)(*it) << " " << (float)score[*it] << endl;
  data_file.close();
};

//--------------------------------------------------
void Analizer::output_PageRank() {
  stringstream filename;
  filename << "network_pageranks-an" << n_nodes << "-" << n_edges << ".dat";
  ofstream data_file(filename.str().c_str());

  ScoreTable score(n_nodes);
  score = calc_PageRank();
  VIter it, it_end;
  for(tie(it, it_end) = vertices(g); it != it_end; ++it) 
    data_file << (int)(*it) << " " << (float) score[*it] << endl;
  data_file.close();
};

//--------------------------------------------------
void Analizer::output_Betweenness() {

  stringstream filename;
  filename << "network_betweenness-an" << n_nodes << "-" << n_edges << ".dat";
  ofstream data_file(filename.str().c_str());

  ScoreTable score(n_nodes);
  score = calc_Betweenness();
  VIter it, it_end;
  for(tie(it, it_end) = vertices(g); it != it_end; ++it)
    data_file << (int)(*it) << " " << (float)score[*it] << endl;
  data_file.close();
  
}

//--------------------------------------------------
ScoreTable Analizer::calc_Closeness() {
  ScoreTable score(num_vertices(g));
  VIter vi, vi_end;

  ScoreTable L(num_vertices(g));
  L = calc_L(g);
  
  for(tie(vi, vi_end) = vertices(g); vi != vi_end; ++vi) {
    score[*vi] = 1 / L[*vi];
  }
  
  return score;

};

//--------------------------------------------------
ScoreTable Analizer::calc_L(const SocialNetwork sn) {
  
  ScoreTable score(num_vertices(sn));
  VIter vi, vi_end;

  for(tie(vi, vi_end) = vertices(sn); vi != vi_end; ++vi) {
    DistTable dist(num_vertices(sn),0);
    breadth_first_search(sn, *vi, visitor(distance_recorder(&dist[0])));
    //dijkstra_shortest_paths(sn, *vi, distance_map(&dist[0]).distance_inf(num_vertices(sn)));
    VIter it, it_end;
    int count = 0;
    for(tie(it, it_end) = vertices(sn); it != it_end; ++it) {
      if(*vi != *it && dist[*it] == 0)
	count += num_vertices(sn);
      else
	count += dist[*it];
      //cout << dist[*it] << " ";
    }
    //cout << endl;
    score[*vi] = (float)count / (float)(num_vertices(sn) - 1);
  }
  return score;

};

//--------------------------------------------------
ScoreTable Analizer::calc_PageRank() {
  VIter vi, vi_end;
  //ScoreTable score(num_vertices(g), 1.0 / (float)num_vertices(g));
  ScoreTable score(num_vertices(g), 1.0);

  while(true) {
    float cache = 0.0;
    ScoreTable nv(num_vertices(g), 0.0);

    for(tie(vi, vi_end) = vertices(g); vi != vi_end; vi++) {
      float num = (float)out_degree(*vi, g);
      if(num == 0.0)
	cache += score[*vi];
      else {
	AIter ai, ai_end;
	for(tie(ai, ai_end) = adjacent_vertices(*vi, g); ai != ai_end; ai++)
	  nv[*ai] += (score[*vi] * 0.85) / num;
	cache += score[*vi] * 0.15;
      }
    }
    float total = 0.0;
    for(tie(vi, vi_end) = vertices(g); vi != vi_end; vi++) {
      nv[*vi] += (cache / (float)num_vertices(g));
      total += (nv[*vi] - score[*vi]) * (nv[*vi] - score[*vi]);
      score[*vi] = nv[*vi];
    }
    //if(total / (float)num_vertices(g) < 0.000001)
    if((total * 1000.0) < 0.0000001)
      break;
  }

  return score;
};

//--------------------------------------------------
ScoreTable Analizer::calc_Betweenness() {

  VIter vi, vi_end, vi2, vi2_end;
  ScoreTable score(num_vertices(g));
  ScoreTable b(num_vertices(g), 0.0);
  
  for(tie(vi, vi_end) = vertices(g); vi != vi_end; vi++) {
    //cout << "vertex:"<< *vi << endl;
    if(out_degree(*vi, g) == 0 ||
       (int)out_degree(*vi, g) == (int)(num_vertices(g) - 1))
      continue;
    DistTable d(num_vertices(g), 0);
    breadth_first_search(g, *vi, visitor(distance_recorder(&d[0])));
    int diameter = 1;
    for(tie(vi2, vi2_end) = vertices(g); vi2 != vi2_end; ++vi2) {
      if(*vi == *vi2 || d[*vi2] == 0)
	continue;
      if(diameter < (int)d[*vi2]) {
	diameter = (int)d[*vi2];
	break;
      }
    }
    if(diameter > 1)
      check_betweenness(1.0, d, b, g, diameter);
  }
  for(tie(vi, vi_end) = vertices(g); vi != vi_end; vi++) {
    //score[*vi] = b[*vi] / 2.0;
    score[*vi] = b[*vi]/ ((((float)num_vertices(g)) * ((float)num_vertices(g) - 2.0)) / 2.0);
  }
  //cout << Abs(((float)num_vertices(g) * ((float)num_vertices(g) - 2.0) / 2.0)) << endl;
  return score;
};

void Analizer::check_betweenness(float value, DistTable d, ScoreTable& b,
			const SocialNetwork sn, int diameter) {
  AIter ai, ai_end;
  map<Vertex, float> betweenness;
  map<int, VerticesL> d_table; // ƱΥˤΡɤޤȤ

   VIter it, it_end;
  for(tie(it, it_end) = vertices(sn); it != it_end; it++){
    if(d[*it] < 1)
      continue;
    d_table[d[*it]].push_back(*it);
    if(diameter < (int)d[*it]) {
      diameter = (int)d[*it];
    }
    //ITK_DBG(d_table[d[*it]].size());
  }

  for(int i = diameter; i > 1; i--) {
    VerticesL vertices = d_table[i];
    while(!vertices.empty()) {
      Vertex v = vertices.front();
      int num = 0;
      for(tie(ai, ai_end) = adjacent_vertices(v, sn); ai != ai_end; ai++)
	if(d[v] > d[*ai])
	  num++;
      for(tie(ai, ai_end) = adjacent_vertices(v, sn); ai != ai_end; ai++) 
	if(d[v] > d[*ai]) {
	  //ITK_DBG(betweenness[v]);
	  //if(i == diameter)
	  //betweenness[*ai] += 1.0 / (float)num;
	  // else
	  //betweenness[*ai] += (betweenness[v]) / (float)num;
	  betweenness[*ai] += (betweenness[v] + 1.0) / (float)num;
	  //ITK_DBG(betweenness[*ai]);
	}
      vertices.pop_front();
    }
  }
  for(tie(it, it_end) = vertices(sn); it != it_end; it++) {
    b[*it] += betweenness[*it];
    //cout << *it << ": " << b[*it] << endl;
  }
}


//--------------------------------------------------
int main(int argc, char* argv[]) {

  using namespace boost;

  std::string filename ="";

  //--------------------------------------------------
  if(argc > 1)
    filename = argv[1];
  else
    return 0;

  GraphvizGraph g_dot;
  boost::read_graphviz(filename, g_dot);

  Analizer analizer;
  SocialNetwork g;

  analizer.init(g_dot);
  //analizer.calc_NetworkProperties();

  analizer.output_DegreeDistribution();
  //analizer.output_Closeness();
  //analizer.output_PageRank();
  //analizer.output_Betweenness();

  return 0;
};
