/* -*- mode: c++ -*-
 * Header    :
 * File      : graphvizDataAnalizer.hxx
 * Auther    : Kosuke Shinoda
 * Since     : <05/06/29 11:28:06 kshinoda>
 * Time-stamp: <08/02/18 12:09:59 kshinoda>
 * Comment   :
 * End       :
 */
/*
 * ScaleFreeNetwork Simulator
 * Copyright (C)2005-2008 SHINODA, Kosuke. NDA, JAPAN
 * Copyright (C)2008 SHINODA, Kosuke. AIST ISRI, JAPAN
 */

#include "common.hxx"
#include <boost/graph/breadth_first_search.hpp>

using namespace boost;

namespace GraphvizDataAnalizer {
  
  class Analizer{
    
    typedef graph_traits<GraphvizGraph>::vertex_iterator GVIter;
    typedef graph_traits<GraphvizGraph>::edge_iterator   GEIter;
    
    //--------------------------------------------------
    /**
   *
   * õ visitor
   */
    template <typename DistanceMap>
    class average_distance_visitor : public default_bfs_visitor {
    
    public:
      average_distance_visitor(DistanceMap dist) : d(dist) {}
      
      template <typename Edge, typename Graph> 
      void tree_edge(Edge e, const Graph& g) const {
	d[target(e,g)] = d[source(e,g)] + 1;
    }
    private:
      DistanceMap d;
    };

    template <typename DistanceMap>
    average_distance_visitor<DistanceMap> distance_recorder(DistanceMap d) {
      return average_distance_visitor<DistanceMap>(d);
    };
    //--------------------------------------------------
    SocialNetwork g;
    
    //--------------------------------------------------
  public:
    Analizer() {};

    int n_nodes;
    int n_edges;
    
  //--------------------------------------------------
  public:
    void init(GraphvizGraph g_dot);
    void calc_NetworkProperties();
    void output_DegreeDistribution();
    void output_Closeness();
    void output_PageRank();
    void output_Betweenness();
  private:
    void calc_ClusteringCoefficient();
    ScoreTable calc_Closeness();
    ScoreTable calc_L(const SocialNetwork sn);
    ScoreTable calc_PageRank();
    ScoreTable calc_Betweenness();
    void check_betweenness(float value, DistTable d, ScoreTable& b,
		     const SocialNetwork sn, int diameter);
  };
};
